package ink.wgink.interfaces.department;

import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.department.DepartmentDTO;
import ink.wgink.pojo.dtos.department.DepartmentSimpleDTO;
import ink.wgink.pojo.pos.DepartmentPO;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IDepartmentService
 * @Description: 组织机构
 * @Author: WangGeng
 * @Date: 2021/1/24 12:53
 * @Version: 1.0
 **/
public interface IDepartmentBaseService {

    /**
     * 部门列表
     *
     * @param params
     * @return
     */
    List<DepartmentDTO> list(Map<String, Object> params);

    /**
     * 部门列表
     *
     * @param ids
     * @return
     */
    List<DepartmentDTO> listByIds(List<String> ids);

    /**
     * 部门列表
     *
     * @param departmentParentId
     * @return
     */
    List<DepartmentDTO> listByParentId(String departmentParentId);

    /**
     * 部门列表
     *
     * @param departmentAreaCode
     * @return
     */
    List<DepartmentDTO> listByAreaCode(String departmentAreaCode);

    /**
     * 部门列表
     *
     * @param departmentAreaCode 地区编码
     * @param order              排序方式
     * @return
     */
    List<DepartmentDTO> listByAreaCodeAndOrder(String departmentAreaCode, String order);

    /**
     * 部门列表
     *
     * @param departmentAreaCode 地区编码
     * @return
     */
    List<DepartmentDTO> listAllByAreaCode(String departmentAreaCode);

    /**
     * 部门列表
     *
     * @param departmentAreaCode 地区编码
     * @param order              排序方式
     * @return
     */
    List<DepartmentDTO> listAllByAreaCodeAndOrder(String departmentAreaCode, String order);

    /**
     * 部门全部子列表
     *
     * @param departmentAreaCode 地区编码
     * @return
     */
    List<DepartmentDTO> listSubAllByAreaCode(String departmentAreaCode);

    /**
     * 部门全部子列表
     *
     * @param departmentAreaCode 地区编码
     * @param order              排序方式
     * @return
     */
    List<DepartmentDTO> listSubAllByAreaCodeAndOrder(String departmentAreaCode, String order);

    /**
     * 通过ID获取组织部门详情
     *
     * @param departmentId
     * @return
     */
    DepartmentDTO get(String departmentId);

    /**
     * easyui组织部门
     *
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentDTO>> listPage(ListPage page);

    /**
     * easyui组织部门列表
     *
     * @param departmentParentId
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentDTO>> listPageByParentId(String departmentParentId, ListPage page);

    /**
     * 组织部门列表
     *
     * @param departmentAreaCode
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentDTO>> listPageByAreaCode(String departmentAreaCode, ListPage page);

    /**
     * 组织部门列表
     *
     * @param departmentAreaCode
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentDTO>> listPageAllByAreaCode(String departmentAreaCode, ListPage page);

    /**
     * 组织部门子列表
     *
     * @param departmentAreaCode
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentDTO>> listPageSubAllByAreaCode(String departmentAreaCode, ListPage page);

    /**
     * 组织部门子列表
     *
     * @param departmentAreaCode 部门地区编码
     * @param type               部门类别
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentDTO>> listPageSubAllByAreaCodeAndType(String departmentAreaCode, Integer departmentType, ListPage page);

    /**
     * 组织部门列表，递归获取全部内容
     *
     * @param params
     * @return
     */
    List<DepartmentDTO> listAll(Map<String, Object> params);

    /**
     * zTree组织部门
     *
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(Map<String, Object> params);

    /**
     * zTree组织部门
     *
     * @param departmentParentId
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(String departmentParentId, Map<String, Object> params);

    /**
     * zTree组织部门
     *
     * @param departmentParentId
     * @param departmentType
     * @return
     */
    List<ZTreeDTO> listZTree(String departmentParentId, Integer departmentType);

    /**
     * zTree组织部门
     *
     * @param areaCode
     * @param parentId
     * @return
     */
    List<ZTreeDTO> listZTreeByAreaCodeAndParentId(String areaCode, String parentId);

    /**
     * 部门列表
     *
     * @param departmentAreaCode 地区ID
     * @param departmentParentId 上级部门ID，0为查询地区下部门，其他为地区下所有下级部门
     * @param departmentType     部门类型，1：机构，2：部门
     * @return
     */
    List<ZTreeDTO> listZTreeByAreaCodeAndParentIdAndType(String departmentAreaCode, String departmentParentId, Integer departmentType);

    /**
     * 部门ID列表
     *
     * @param params
     * @return
     */
    List<String> listId(Map<String, Object> params);

    /**
     * 获取基础部门ID列表
     *
     * @param departmentSimpleDTOs
     * @return
     */
    List<String> listRootId(List<DepartmentSimpleDTO> departmentSimpleDTOs);

    /**
     * 获取上级部门
     *
     * @param departmentSimpleDTO1
     * @param departmentSimpleDTO2
     * @return
     */
    DepartmentSimpleDTO getHigherLevel(DepartmentSimpleDTO departmentSimpleDTO1, DepartmentSimpleDTO departmentSimpleDTO2);

    /**
     * 获得同一级部门数量（按部门名称）
     *
     * @param params
     * @return
     */
    Integer count(Map<String, Object> params);

    /**
     * 通过ID获取组织部门详情（简单格式）
     *
     * @param departmentId
     * @return
     */
    DepartmentPO getPO(String departmentId);

    /**
     * 部门列表
     *
     * @param params
     * @return
     */
    List<DepartmentPO> listPO(Map<String, Object> params);

    /**
     * 部门列表（简单格式）
     *
     * @param departmentIds 部门ID列表
     * @return
     */
    List<DepartmentPO> listPO(List<String> departmentIds);

    /**
     * 通过上级ID获取组织部门全部列表（简单格式）
     *
     * @param departmentParentId
     * @return
     */
    List<DepartmentPO> listPOByParentId(String departmentParentId);

    /**
     * 通过上级ID和添加获取组织部门全部增量列表（简单格式）
     *
     * @param departmentParentId
     * @param addDate
     * @return
     */
    List<DepartmentPO> listPOByParentIdAndAddDate(String departmentParentId, String addDate);

    /**
     * 通过上级ID和更新获取组织部门全部新增和修改的列表（简单格式）
     *
     * @param departmentParentId
     * @param updateDate
     * @return
     */
    List<DepartmentPO> listPOByParentIdAndUpdateDate(String departmentParentId, String updateDate);

    /**
     * 通过区域编码获取组织部门列表
     *
     * @param departmentAreaCode 部门区域编码
     * @return
     */
    List<DepartmentPO> listPOByAreaCode(String departmentAreaCode);

    /**
     * 通过区域编码、部门类型获取组织部门列表
     *
     * @param departmentAreaCode
     * @param departmentType
     * @return
     */
    List<DepartmentPO> listPOByAreaCodeAndType(String departmentAreaCode, Integer departmentType);

    /**
     * 统计地区全部组织
     *
     * @param areaCode
     * @return
     */
    Integer countByAreaCode(String areaCode);

    /**
     * 统计地区全部组织
     *
     * @param areaCode
     * @return
     */
    Integer countAllByAreaCode(String areaCode);

}
