package ink.wgink.interfaces.department;

import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.department.DepartmentDTO;
import ink.wgink.pojo.dtos.department.DepartmentUserDTO;
import ink.wgink.pojo.dtos.user.UserDTO;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IDepartmentUser
 * @Description: 部门用户
 * @Author: WangGeng
 * @Date: 2021/1/24 12:59
 * @Version: 1.0
 **/
public interface IDepartmentUserBaseService {

    /**
     * 机构用户列表
     *
     * @param params
     * @return
     */
    List<DepartmentUserDTO> list(Map<String, Object> params);

    /**
     * 机构用户列表
     *
     * @param departmentId
     * @return
     */
    List<DepartmentUserDTO> list(String departmentId);

    /**
     * 机构用户列表
     *
     * @param departmentIds
     * @return
     */
    List<DepartmentUserDTO> list(List<String> departmentIds);

    /**
     * 机构用户列表
     *
     * @param userIds
     * @return
     */
    List<DepartmentUserDTO> listByUserIds(List<String> userIds);

    /**
     * 用户ID列表
     *
     * @param departmentId 部门ID
     * @return
     */
    List<String> listUserId(String departmentId);


    /**
     * 用户ID列表
     *
     * @param departmentIds 部门ID列表
     * @return
     */
    List<String> listUserId(List<String> departmentIds);

    /**
     * 用户ID列表
     *
     * @param departmentId 部门ID
     * @param userIds      用户ID列表
     * @return
     */
    List<String> listUserId(String departmentId, List<String> userIds);

    /**
     * 用户ID列表
     *
     * @param departmentIds 部门ID列表
     * @param userIds       用户ID列表
     * @return
     */
    List<String> listUserId(List<String> departmentIds, List<String> userIds);

    /**
     * 用户列表
     *
     * @param departmentId 部门ID
     * @param page
     * @return
     */
    SuccessResultList<List<DepartmentUserDTO>> listPage(String departmentId, ListPage page);

    /**
     * 用户列表
     *
     * @param departmentId 部门ID
     * @param params       参数
     * @return
     */
    List<DepartmentUserDTO> list(String departmentId, Map<String, Object> params);

    /**
     * 设置用户
     *
     * @param departmentUserDTOs
     */
    void setUser(List<DepartmentUserDTO> departmentUserDTOs);

    /**
     * 部门列表
     *
     * @return
     */
    List<DepartmentDTO> listDepartmentBySelf();

    /**
     * 部门列表
     *
     * @param userId 用户ID
     * @return
     */
    List<DepartmentDTO> listDepartmentByUserId(String userId);

    /**
     * 用户分页列表
     *
     * @param departmentId
     * @return
     */
    List<UserDTO> listUserByDepartmentId(String departmentId);

    /**
     * 用户分页列表
     *
     * @param departmentId 部门ID
     * @param page
     * @return
     */
    SuccessResultList<List<UserDTO>> listPageUserByDepartmentId(String departmentId, ListPage page);

    /**
     * 部门ID列表
     *
     * @param userId 用户ID
     * @return
     */
    List<String> listDepartmentIdByUserId(String userId);

    /**
     * 部门ID列表
     *
     * @param userIds 用户ID列表
     * @return
     */
    List<String> listDepartmentIdByUserIds(List<String> userIds);

    /**
     * 部门用户ID列表
     *
     * @param userId 用户ID
     * @return
     */
    List<String> listSameDepartmentUserIdByUserId(String userId);

    /**
     * 部门用户ID列表
     *
     * @param userIds 用户ID列表
     * @return
     */
    List<String> listSameDepartmentUserIdByUserIds(List<String> userIds);

}
