package ink.wgink.interfaces.position;

import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.position.PositionDTO;
import ink.wgink.pojo.pos.PositionPO;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IPositionService
 * @Description: 职位
 * @Author: WangGeng
 * @Date: 2021/1/24 12:54
 * @Version: 1.0
 **/
public interface IPositionBaseService {

    /**
     * 全部职位JSON列表
     *
     * @param positionParentId
     * @return
     */
    List<PositionDTO> listAllByParentId(String positionParentId);

    /**
     * 职位列表，递归获取全部内容
     *
     * @param params
     * @return
     */
    List<PositionDTO> listAll(Map<String, Object> params);

    /**
     * 职位详情
     *
     * @param params
     * @return
     */
    PositionDTO get(Map<String, Object> params);

    /**
     * 职位详情
     *
     * @param positionId
     * @return
     */
    PositionDTO get(String positionId);

    /**
     * 职位zTree列表
     *
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(Map<String, Object> params);

    /**
     * 职位EasyUI列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<PositionDTO>> listPage(ListPage page);

    /**
     * 职位列表
     *
     * @param params
     * @return
     */
    List<PositionDTO> list(Map<String, Object> params);

    /**
     * 职位列表
     *
     * @param positionIds 用户ID列表
     * @return
     */
    List<PositionDTO> listByPositionIds(List<String> positionIds);

    /**
     * 职位列表
     *
     * @param params
     * @return
     */
    List<PositionPO> listPO(Map<String, Object> params);

    /**
     * 职位列表
     *
     * @param positionIds
     * @return
     */
    List<PositionPO> listPO(List<String> positionIds);

    /**
     * 统计
     *
     * @param params
     * @return
     */
    Integer count(Map<String, Object> params);

    /**
     * 上级详情
     *
     * @param positionId 职位ID
     * @return
     */
    PositionDTO getParentByPositionId(String positionId);

    /**
     * 上级列表
     *
     * @param positionIds 职位ID列表
     * @return
     */
    List<PositionDTO> listParentByPositionIds(List<String> positionIds);

    /**
     * 职位列表
     *
     * @param positionParentIds 上级职位ID
     * @return
     */
    List<PositionDTO> listByPositionParentIds(List<String> positionParentIds);
}
