package ink.wgink.interfaces.role;

import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.role.RoleSimpleDTO;
import ink.wgink.pojo.dtos.role.RoleUserDTO;
import ink.wgink.pojo.dtos.user.UserDTO;
import ink.wgink.pojo.pos.RolePO;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IRoleUserBaseService
 * @Description: 角色用户
 * @Author: WangGeng
 * @Date: 2021/2/18 11:36
 * @Version: 1.0
 **/
public interface IRoleUserBaseService {

    /**
     * 用户ID列表
     *
     * @param roleId 用户组ID
     * @return
     */
    List<String> listUserId(String roleId);

    /**
     * 用户ID列表
     *
     * @param roleIds 用户组ID列表
     * @return
     */
    List<String> listUserId(List<String> roleIds);

    /**
     * 分页用户组用户
     *
     * @param roleId
     * @param page
     * @return
     */
    SuccessResultList<List<RoleUserDTO>> listPage(String roleId, ListPage page);

    /**
     * 角色列表
     *
     * @param userId
     * @return
     */
    List<RolePO> listRolePOByUserId(String userId);

    /**
     * 角色列表
     *
     * @param roleIds
     * @return
     */
    List<RolePO> listRolePOByRoleIds(List<String> roleIds);

    /**
     * 角色列表（简单）
     *
     * @param rolePOs
     * @return
     */
    List<RoleSimpleDTO> listSimple(List<RolePO> rolePOs);

    /**
     * 用户ID列表
     *
     * @param roleId  角色ID
     * @param userIds 用户ID列表
     * @return
     */
    List<String> listUserIdByRoleIdAndUserIds(String roleId, List<String> userIds);

    /**
     * 用户ID列表
     * @param roleIds 角色ID列表
     * @param userIds 用户ID列表
     * @return
     */
    List<String> listUserIdByRoleIdsAndUserIds(List<String> roleIds, List<String> userIds);

    /**
     * 角色ID列表
     *
     * @param userId
     * @return
     */
    List<String> listRoleIdByUserId(String userId);

    /**
     * 用户列表
     *
     * @param roleId 角色ID
     * @return
     */
    List<UserDTO> listUserByRoleId(String roleId);

    /**
     * 用户分页列表
     *
     * @param roleId
     * @param page
     * @return
     */
    SuccessResultList<List<UserDTO>> listPageUserByRoleId(String roleId, ListPage page);

    /**
     * 无部门无角色用户
     *
     * @param params
     * @return
     */
    List<UserDTO> listUserNoDepartmentAndNoRole(Map<String, Object> params);

    /**
     * 没有角色并且没有组织机构的用户分页列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<UserDTO>> listPageUserNoDepartmentAndNoRole(ListPage page);

}
