package ink.wgink.module.dictionary.service;

import ink.wgink.module.dictionary.pojo.dtos.AreaDTO;
import ink.wgink.module.dictionary.pojo.vos.AreaVO;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IAreaAreaService
 * @Description: 地区字典
 * @Author: WangGeng
 * @Date: 2019/11/18 14:09
 * @Version: 1.0
 **/
public interface IAreaService {

    /**
     * 地区字典新增
     *
     * @param areaVO
     * @return
     * @throws Exception
     */
    void save(AreaVO areaVO);

    /**
     * 地区字典新增
     *
     * @param areaVO
     * @return 新增ID
     * @throws Exception
     */
    String saveReturnId(AreaVO areaVO);

    /**
     * 地区字典删除
     *
     * @param ids
     * @return
     */
    void remove(List<String> ids);

    /**
     * 地区字典修改
     *
     * @param areaId
     * @param areaVO
     * @return
     */
    void update(String areaId, AreaVO areaVO);

    /**
     * 通过ID获取地区字典
     *
     * @param areaId
     * @return
     */
    AreaDTO get(String areaId);

    /**
     * 通过编码获取地区字典详情
     *
     * @param areaCode
     * @return
     */
    AreaDTO getByCode(String areaCode);

    /**
     * 地区列表
     *
     * @return
     */
    List<AreaDTO> list();

    /**
     * 通过上级ID获取地区字典列表
     *
     * @param areaParentId
     * @return
     */
    List<AreaDTO> listByParentId(String areaParentId);

    /**
     * 通过上级ID获取地区字典全部列表
     *
     * @param areaParentId
     * @return
     */
    List<AreaDTO> listAllByParentId(String areaParentId);

    /**
     * 分页获取地区字典列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<AreaDTO>> listPage(ListPage page);

    /**
     * zTree列表
     *
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(Map<String, Object> params);

    /**
     * 地区字典列表（通过地区字典编码）
     *
     * @param areaCode 地区编码
     * @return
     */
    List<AreaDTO> listByCode(String areaCode);

    /**
     * 地区字典子列表（不包含本级）
     *
     * @param areaCode 地区编码
     * @return
     */
    List<AreaDTO> listSubByCode(String areaCode);

}
