package ink.wgink.module.dictionary.service;

import ink.wgink.interfaces.dictionary.IDictionaryCheckService;
import ink.wgink.module.dictionary.pojo.dtos.DataDTO;
import ink.wgink.module.dictionary.pojo.vos.DataVO;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IDictionaryService
 * @Description: 字典
 * @Author: WangGeng
 * @Date: 2019/11/18 14:09
 * @Version: 1.0
 **/
public interface IDataService extends IDictionaryCheckService {
    /**
     * 字典新增
     *
     * @param dataVO
     * @return
     */
    void save(DataVO dataVO);

    /**
     * 字典新增
     *
     * @param dataVO
     * @return 新增ID
     */
    String saveReturnId(DataVO dataVO);

    /**
     * 字典删除
     *
     * @param ids
     * @return
     */
    void remove(List<String> ids);

    /**
     * 字典修改
     *
     * @param dataId
     * @param dataVO
     * @return
     */
    void update(String dataId, DataVO dataVO);

    /**
     * 通过ID获取字典
     *
     * @param dataId
     * @return
     */
    DataDTO get(String dataId);

    /**
     * 字典列表
     *
     * @return
     */
    List<DataDTO> list();

    /**
     * 通过上级ID获取字典列表
     *
     * @param dataParentId
     * @return
     */
    List<DataDTO> listByParentId(String dataParentId);

    /**
     * 通过上级ID获取字典全部列表
     *
     * @param dataParentId
     * @return
     */
    List<DataDTO> listAllByParentId(String dataParentId);

    /**
     * 分页获取字典列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<DataDTO>> listPage(ListPage page);

    /**
     * zTree列表
     *
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(Map<String, Object> params);

    /**
     * 字典列表（通过字典编码）
     *
     * @param dataCode
     * @return
     */
    List<DataDTO> listByCode(String dataCode);

}
