function FormUtil(layui, viewer) {
    var $ = layui.$;
    var layer = layui.layer;
    var upload = layui.upload;
    var laydate = layui.laydate;
    var form = layui.form;
    var Viewer = viewer;
    var restAjax = layui.restajax;
    var viewerObj = {};

    /**
     * 初始化图片上传
     * @param fieldName
     * @param maxCount
     */
    this.initUploadImage = function (fieldName, maxCount) {
        maxCount = maxCount ? parseInt(maxCount) : 9
        maxCount = maxCount < 0 ? 9 : maxCount;
        var id = '#' + fieldName;
        var fileBoxId = id + 'FileBox';
        var uploadBtnId = id + 'UploadBtn';
        var deleteBtnClass = '.delete-' + fieldName + '-btn';

        function init(callback) {
            var fileIds = $(id).val();
            var fileIdArray = fileIds ? fileIds.split(',') : [];

            var html = '';
            for (var i = 0, fileId; fileId = fileIdArray[i++];) {
                html += [
                    '<div class="form-upload-image">',
                    '   <img src="route/file/download/true/' + fileId + '" align="加载失败">',
                    '   <a class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-' + fieldName + '-btn" href="javascript:void(0);" data-id="' + fileId + '" data-name="' + fieldName + '">',
                    '       <i class="fa fa-trash-o"></i>',
                    '   </a>',
                    '</div>',
                ].join('');
            }

            $(fileBoxId).empty();
            $(fileBoxId).append(html);

            if (fileIdArray.length < maxCount) {
                $(uploadBtnId).removeClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', false);
            } else {
                $(uploadBtnId).addClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', true);
            }

            callback ? callback() : '';
        }

        function addClick() {
            upload.render({
                elem: uploadBtnId,
                url: 'api/file/v2/upload-image',
                accept: 'images',
                acceptMime: 'image/*',
                field: 'image',
                exts: 'jpg|png|gif|bmp|jpeg',
                name: fieldName,
                done: function (res, index, upload) {
                    var name = this.name;
                    var files = $('#' + this.name).val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId;
                    $('#' + name).val(files);
                    init(function () {
                        viewerObj[name].update();
                    });
                },
                error: function (index, upload) {
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                    var percent = n + '%'
                    console.log(percent);
                    console.log(elem);
                    console.log(res);
                    console.log(index);
                    element.progress('demo-' + index, n + '%');
                }
            });
            $(document).on('click', deleteBtnClass, function () {
                var name = this.dataset.name;
                var id = this.dataset.id;
                var files = $('#' + name).val().replace(id, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) == ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) == ',') {
                    files = files.substring(0, files.length - 1);
                }
                $('#' + name).val(files);
                init(function () {
                    viewerObj[name].update();
                });
            });
        }

        init(function () {
            viewerObj[fieldName] = new Viewer(document.getElementById(fieldName + 'FileBox'), {navbar: false});
        });
        addClick();
    }

    /**
     * 初始化文件上传
     * @param fieldName
     * @param maxCount
     */
    this.initUploadFile = function (fieldName, maxCount) {
        maxCount = maxCount ? parseInt(maxCount) : 3
        maxCount = maxCount < 0 ? 3 : maxCount;
        var id = '#' + fieldName;
        var fileBoxId = id + 'FileBox';
        var uploadBtnId = id + 'UploadBtn';
        var deleteBtnClass = '.delete-' + fieldName + '-btn';

        function init() {
            var files = $(id).val();
            var fileArray = files ? files.split(',') : [];

            var html = '';
            if (fileArray.length > 0) {
                html += [
                    '<div class="form-upload-file">',
                    '   <table class="layui-table" lay-size="sm">',
                    '       <colgroup>',
                    '           <col>',
                    '           <col width="60">',
                    '       </colgroup>',
                    '       <thead>',
                    '       <tr>',
                    '           <th>文件名</th>',
                    '           <th class="operation">操作</th>',
                    '       </tr>',
                    '       </thead>',
                    '       <tbody>',
                ].join('');
                for (var i = 0, file; file = fileArray[i++];) {
                    var idNameArray = file.split(':');
                    var fileId = idNameArray[0];
                    var fileName = idNameArray[1];
                    html += [
                        '<tr>',
                        '   <td>' + fileName + '</td>',
                        '   <td class="operation">',
                        '       <button type="button" class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-' + fieldName + '-btn" data-id="' + fileId + '" data-name="' + fileName + '" data-field-name="' + fieldName + '">删除</button>',
                        '   </td>',
                        '</tr>',
                    ].join('');
                }
                html += [
                    '       </tbody>',
                    '   </table>',
                    '</div>'
                ].join('');
            }

            $(fileBoxId).empty();
            $(fileBoxId).append(html);

            if (fileArray.length < maxCount) {
                $(uploadBtnId).removeClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', false);
            } else {
                $(uploadBtnId).addClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', true);
            }
        }

        function addClick() {
            upload.render({
                elem: uploadBtnId,
                url: 'api/file/v2/upload-file',
                accept: 'file',
                acceptMime: [
                    'application/pdf',
                    'application/msword',
                    'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
                    'application/vnd.ms-excel',
                    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
                    'application/vnd.ms-powerpoint',
                    'application/vnd.openxmlformats-officedocument.presentationml.presentation',
                    'application/vnd.ms-works',
                    'text/plain',
                    'application/x-rar',
                    'application/x-zip-compressed'
                ].join(''),
                field: 'file',
                exts: 'pdf|doc|docx|xls|xlsx|ppt|pptx|wps|txt|rar|zip',
                name: fieldName,
                done: function (res, index, upload) {
                    var name = this.name;
                    var files = $('#' + this.name).val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId + ':' + res.data.fileName;
                    $('#' + name).val(files);
                    init();
                },
                error: function (index, upload) {
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                    var percent = n + '%'
                    console.log(percent);
                    console.log(elem);
                    console.log(res);
                    console.log(index);
                    element.progress('demo-' + index, n + '%');
                }
            });
            $(document).on('click', deleteBtnClass, function () {
                var id = this.dataset.id;
                var name = this.dataset.name;
                var fieldName = this.dataset.fieldName;
                var files = $('#' + fieldName).val().replace(id + ':' + name, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) == ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) == ',') {
                    files = files.substring(0, files.length - 1);
                }
                $('#' + fieldName).val(files);
                init();
            });
        }

        init();
        addClick();
    }

    /**
     * 初始化视频上传
     * @param fileName
     * @param maxCount
     */
    this.initUploadVideo = function (fileName, maxCount) {
        maxCount = maxCount ? parseInt(maxCount) : 1
        maxCount = maxCount < 0 ? 1 : maxCount;
        var id = '#' + fileName;
        var fileBoxId = id + 'FileBox';
        var uploadBtnId = id + 'UploadBtn';
        var deleteBtnClass = '.delete-' + fileName + '-btn';

        function init() {
            var fileIds = $(id).val();
            var fileIdArray = fileIds ? fileIds.split(',') : [];
            var html = '';
            for (var i = 0, fileId; fileId = fileIdArray[i++];) {
                html += [
                    '<div class="form-upload-video">',
                    '   <video width="320" height="240" controls>',
                    '       <source src="route/file/download/true/' + fileId + '" type="video/mp4">',
                    '       您的浏览器不支持 video 标签',
                    '   </video>',
                    '   <a class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-' + fileName + '-btn" href="javascript:void(0);" data-id="' + fileId + '" data-name="' + fileName + '">',
                    '       <i class="fa fa-trash-o"></i>',
                    '   </a>',
                    '</div>',
                ].join('');
            }

            $(fileBoxId).empty();
            $(fileBoxId).append(html);

            if (fileIdArray.length < maxCount) {
                $(uploadBtnId).removeClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', false);
            } else {
                $(uploadBtnId).addClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', true);
            }
        }

        function addClick() {
            upload.render({
                elem: uploadBtnId,
                url: 'api/file/v2/upload-video',
                accept: 'video',
                acceptMime: 'video/mp4',
                exts: 'mp4',
                field: 'video',
                name: fileName,
                done: function (res, index, upload) {
                    var name = this.name;
                    var files = $('#' + this.name).val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId;
                    $('#' + name).val(files);
                    init();
                },
                error: function (index, upload) {
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                    var percent = n + '%'
                    console.log(percent);
                    console.log(elem);
                    console.log(res);
                    console.log(index);
                    element.progress('demo-' + index, n + '%');
                }
            });
            $(document).on('click', deleteBtnClass, function () {
                var name = this.dataset.name;
                var id = this.dataset.id;
                var files = $('#' + name).val().replace(id, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) == ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) == ',') {
                    files = files.substring(0, files.length - 1);
                }
                $('#' + name).val(files);
                init();
            });
        }

        init();
        addClick();
    }

    /**
     * 初始化音频上传
     * @param fileName
     * @param maxCount
     */
    this.initUploadAudio = function (fileName, maxCount) {
        maxCount = maxCount ? parseInt(maxCount) : 1
        maxCount = maxCount < 0 ? 1 : maxCount;
        var id = '#' + fileName;
        var fileBoxId = id + 'FileBox';
        var uploadBtnId = id + 'UploadBtn';
        var deleteBtnClass = '.delete-' + fileName + '-btn';

        function init() {
            var fileIds = $(id).val();
            var fileIdArray = fileIds ? fileIds.split(',') : [];
            var html = '';
            for (var i = 0, fileId; fileId = fileIdArray[i++];) {
                html += [
                    '<div class="form-upload-video">',
                    '   <audio src="route/file/download/true/' + fileId + '" controls>',
                    '       您的浏览器不支持 audio 标签。',
                    '   </audio>',
                    '   <a class="layui-btn layui-btn-xs layui-btn-danger delete-btn delete-' + fileName + '-btn" href="javascript:void(0);" data-id="' + fileId + '" data-name="' + fileName + '">',
                    '       <i class="fa fa-trash-o"></i>',
                    '   </a>',
                    '</div>',
                ].join('');
            }

            $(fileBoxId).empty();
            $(fileBoxId).append(html);

            if (fileIdArray.length < maxCount) {
                $(uploadBtnId).removeClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', false);
            } else {
                $(uploadBtnId).addClass('layui-btn-disabled');
                $(uploadBtnId).attr('disabled', true);
            }
        }

        function addClick() {
            upload.render({
                elem: uploadBtnId,
                url: 'api/file/v2/upload-audio',
                accept: 'audio',
                acceptMime: [
                    'audio/wav',
                    'audio/mp3'
                ],
                exts: 'wav|mp3',
                field: 'audio',
                name: fileName,
                done: function (res, index, upload) {
                    var name = this.name;
                    var files = $('#' + this.name).val();
                    if (files.length > 0) {
                        files += ',';
                    }
                    files += res.data.fileId;
                    $('#' + name).val(files);
                    init();
                },
                error: function (index, upload) {
                    layer.msg('文件上传失败');
                },
                progress: function (n, elem, res, index) {
                    var percent = n + '%'
                    console.log(percent);
                    console.log(elem);
                    console.log(res);
                    console.log(index);
                    element.progress('demo-' + index, n + '%');
                }
            });
            $(document).on('click', deleteBtnClass, function () {
                var name = this.dataset.name;
                var id = this.dataset.id;
                var files = $('#' + name).val().replace(id, '');
                files = files.replace(/\,+/g, ',');
                if (files.charAt(0) == ',') {
                    files = files.substring(1);
                }
                if (files.charAt(files.length - 1) == ',') {
                    files = files.substring(0, files.length - 1);
                }
                $('#' + name).val(files);
                init();
            });
        }

        init();
        addClick();
    }

    /**
     * 初始化日期
     * @param opt
     */
    this.initDate = function (opt) {
        laydate.render({
            elem: '#' + opt.id,
            type: opt.datetype,
            format: opt.dateformat,
            value: opt.dateDefaultValue,
            min: opt.dataMinValue,
            max: opt.dataMaxValue,
            trigger: 'click'
        });
    }

    /**
     * 初始化checkbox数据
     * @param form
     * @param checkedData
     * @param id
     */
    this.initCheckboxData = function (formName, checkedData, id) {
        var dataArray = checkedData.split(',');
        var obj = {};
        for (var i = 0, data; data = dataArray[i++];) {
            obj[id + '[' + data + ']'] = true;
        }
        form.val(formName, obj);
    }

    /**
     * 设置复选框值
     * @param formData
     * @param id
     */
    this.setCheckboxValue = function (formData, id) {
        formData.field[id] = restAjax.checkBoxToString(formData.field, id);
    }

    /**
     * 设置开关值
     * @param formData
     * @param id
     */
    this.setSwitchValue = function (formData, id) {
        formData.field[id] = formData.field[id] ? 1 : 0;
    }

    /**
     * 清空上传字段
     * @param formData
     */
    this.clearUploadField = function (formData) {
        var fileInputs = $(formData.form).find('input[type="file"]');
        if (fileInputs.length == 0) {
            return;
        }
        $.each(fileInputs, function (index, item) {
            delete formData.field[item.name];
        });
    }

    /**
     * @callback onConfirm
     * @param selectedUsers {Array} [{userId: '', userName: '', userUsername: ''}]
     */
    /**
     * @description 选择用户列表
     * @param opt {object}
     * @param {String} opt.selectType 'radio|checkbox' 必传，2选1
     * @param {onConfirm} opt.onConfirm 确定按钮回调函数，必传
     * @param {Array} opt.users 参与选择的用户列表，必传，格式：[{userId: '', userUsername: '', userName: ''}]
     * @param {Array} opt.selectedUserIds 默认选中的用户ID列表，非必传，如果selectType为 radio，默认选中[0]。格式：['userId1', 'userId2']
     */
    this.selectUsers = function (opt) {
        var selectType = opt.selectType,
            onConfirm = opt.onConfirm,
            users = opt.users,
            selectedUserIds = opt.selectedUserIds;
        if (selectType != 'radio' && selectType != 'checkbox') {
            throw 'selectType(arg1): [radio|checkbox]';
            return;
        }
        if (!users && !(users instanceof Array)) {
            onConfirm([]);
        }

        selectedUserIds = selectedUserIds && (selectedUserIds instanceof Array) ? selectedUserIds : [];
        selectedUserIds = selectType == 'radio' && selectedUserIds.length > 1 ? selectedUserIds[0] : selectedUserIds;

        var selectedUserMap = {};
        var selectedUsers = [];
        var itemHtml = '';

        function addUserToMap(userId, userUsername, userName) {
            selectedUserMap[userId] = {
                userId: userId,
                userUsername: userUsername,
                userName: userName,
            }
        }

        function addClick() {
            $('.select-user .list .item').click(function () {
                var dataset = this.dataset;
                var userId = dataset.userId;
                if (selectType === 'radio') {
                    $('.select-user .list .item').removeClass('active');
                    $(this).addClass('active');
                    selectedUserMap = {};
                    addUserToMap(userId, dataset.userUsername, dataset.userName);
                } else {
                    if (selectedUserMap[userId]) {
                        delete selectedUserMap[userId];
                        $(this).removeClass('active');
                    } else {
                        addUserToMap(userId, dataset.userUsername, dataset.userName);
                        $(this).addClass('active');
                    }
                }
            });
            $('.select-user .foot .confirm-btn').click(function () {
                for (var key in selectedUserMap) {
                    selectedUsers.push(selectedUserMap[key]);
                }
                if (selectedUsers.length == 0) {
                    layer.msg('请选择人员');
                    return;
                }
                layer.close(layIndex);
            });
            $('.select-user .foot .close-btn').click(function () {
                selectedUsers = null;
                layer.close(layIndex);
            });
        }

        // 添加用户到map
        for (var i = 0, user; user = users[i++];) {
            var isActive = false;
            for (var j = 0, selectedUserId; selectedUserId = selectedUserIds[j++];) {
                if (user.userId === selectedUserId) {
                    isActive = true;
                    addUserToMap(user.userId, user.userUsername, user.userName);
                    break;
                }
            }
            itemHtml += '<div class="item' + (isActive ? ' active' : '') + '" data-user-id="' + user.userId + '" data-user-username="' + user.userUsername + '" data-user-name="' + user.userName + '">';
            itemHtml += '<div class="avatar">';
            if (user.avatar) {
                itemHtml += '<img src="route/file/download/true/' + user.avatar + '"/>';
            } else {
                itemHtml += '<img src="assets/images/profile-photo.jpg"/>';
            }
            itemHtml += '</div>';
            itemHtml += '<div class="info">';
            itemHtml += '<div class="text">' + user.userUsername + '</div>';
            itemHtml += '<div class="text">' + user.userName + '</div>';
            itemHtml += '</div>';
            itemHtml += '<div class="checkbox">';
            itemHtml += '<i class="fa fa-check-square-o"></i>';
            itemHtml += '</div>';
            itemHtml += '</div>';
        }

        var layIndex = layer.open({
            type: 1,
            area: ['300px', '400px'],
            closeBtn: 0,
            title: '选择候选人',
            shadeClose: false,
            scrollbar: false,
            content: '<div class="select-user">' +
                '<div class="list">'
                + itemHtml +
                '</div>' +
                '<div class="foot">' +
                '<div class="layui-btn-group">' +
                '<button type="button" class="layui-btn layui-btn-sm confirm-btn">确定</button>' +
                '<button type="button" class="layui-btn layui-btn-sm layui-btn-primary close-btn">关闭</button>' +
                '</div>' +
                '</div>' +
                '</div>',
            end: function () {
                onConfirm(selectedUsers);
            }
        });

        addClick();
    }

    /**
     * @description 禁止字段操作
     * @param fields {Array} [{fieldName:'', isEditable: 0|1}]
     */
    this.disableFields = function (fields) {
        for (var i = 0, item; item = fields[i++];) {
            if (parseInt(item.isEditable) === 0) {
                var boxId = '#' + item.fieldName + 'Box';
                $(boxId).css('position', 'relative');
                $(boxId).append('<div style="width: 100%; height: 100%; background-color: #fff0; position: absolute; top: 0; left: 0; cursor: not-allowed;"></div>');
            }
        }
    }

    /**
     * @description 隐藏字段操作
     * @param fields {Array} [{fieldName:'', isVisible: 0|1}]
     */
    this.hideFields = function (fields) {
        for (var i = 0, item; item = fields[i++];) {
            if (parseInt(item.isVisible) === 0) {
                var boxId = '#' + item.fieldName + 'Box';
                $(boxId).hide();
            }
        }
    }


}