package ink.wgink.module.map.controller.app.api.userlocation;

import ink.wgink.annotation.CheckRequestBodyAnnotation;
import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.module.map.pojo.dtos.userlocation.UserLocationDTO;
import ink.wgink.module.map.pojo.vos.userlocation.UserLocationVO;
import ink.wgink.module.map.service.userlocation.IUserLocationService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: UserLocationAppController
 * @Description: 用户定位
 * @Author: CodeFactory
 * @Date: 2021-10-29 15:37:01
 * @Version: 3.0
 **/
@Api(tags = ISystemConstant.API_TAGS_APP_PREFIX + "用户定位接口")
@RestController
@RequestMapping(ISystemConstant.APP_PREFIX + "/user-location")
public class UserLocationAppController extends DefaultBaseController {

    @Autowired
    private IUserLocationService userLocationService;

    @ApiOperation(value = "新增用户定位", notes = "新增用户定位接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("save")
    @CheckRequestBodyAnnotation
    public SuccessResult save(@RequestHeader("token") String token, @RequestBody UserLocationVO userLocationVO) {
        userLocationService.save(token, userLocationVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "删除用户定位(id列表)", notes = "删除用户定位(id列表)接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
        @ApiImplicitParam(name = "ids", value = "ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("remove/{ids}")
    public SuccessResult remove(@RequestHeader("token") String token, @PathVariable("ids") String ids) {
        userLocationService.remove(token, Arrays.asList(ids.split("\\_")));
        return new SuccessResult();
    }

    @ApiOperation(value = "修改用户定位", notes = "修改用户定位接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
        @ApiImplicitParam(name = "userLocationId", value = "用户定位ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("updateuserlocation/{userLocationId}")
    @CheckRequestBodyAnnotation
    public SuccessResult updateUserLocation(@RequestHeader("token") String token, @PathVariable("userLocationId") String userLocationId, @RequestBody UserLocationVO userLocationVO) {
        userLocationService.update(token, userLocationId, userLocationVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "用户定位详情(通过ID)", notes = "用户定位详情(通过ID)接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
        @ApiImplicitParam(name = "userLocationId", value = "用户定位ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get/{userLocationId}")
    public UserLocationDTO get(@RequestHeader("token") String token, @PathVariable("userLocationId") String userLocationId) {
        return userLocationService.get(userLocationId);
    }

    @ApiOperation(value = "用户定位列表", notes = "用户定位列表接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("list")
    public List<UserLocationDTO> list(@RequestHeader("token") String token) {
        Map<String, Object> params = requestParams();
        return userLocationService.list(params);
    }

    @ApiOperation(value = "用户定位分页列表", notes = "用户定位分页列表接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
        @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
        @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
        @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
        @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
        @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpageuserlocation")
    public SuccessResultList<List<UserLocationDTO>> listPage(@RequestHeader("token") String token, ListPage page) {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return userLocationService.listPage(page);
    }

    @ApiOperation(value = "用户定位统计", notes = "用户定位统计接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("count")
    SuccessResultData<Integer> count() {
        Map<String, Object> params = requestParams();
        return new SuccessResultData<>(userLocationService.count(params));
    }

}