package ink.wgink.mongo.module.dictionary.service;

import ink.wgink.module.dictionary.pojo.dtos.AreaDTO;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IAreaAreaService
 * @Description: 地区字典
 * @Author: WangGeng
 * @Date: 2019/11/18 14:09
 * @Version: 1.0
 **/
public interface IMongoAreaService {

    /**
     * 更新mongo
     */
    void updateMongo();

    /**
     * 通过ID获取地区字典
     *
     * @param areaId
     * @return
     */
    AreaDTO get(String areaId);

    /**
     * 通过编码获取地区字典详情
     *
     * @param areaCode
     * @return
     */
    AreaDTO getByCode(String areaCode);

    /**
     * 通过上级ID获取地区字典列表
     *
     * @param areaParentId
     * @return
     */
    List<AreaDTO> listByParentId(String areaParentId);

    /**
     * 通过上级ID获取地区字典全部列表
     *
     * @param areaParentId
     * @return
     */
    List<AreaDTO> listAllByParentId(String areaParentId);

    /**
     * 分页获取地区字典列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<AreaDTO>> listPage(ListPage page);

    /**
     * zTree列表
     *
     * @param areaParentId
     * @return
     */
    List<ZTreeDTO> listZTree(String areaParentId);

    /**
     * 地区字典列表（通过地区字典编码）
     *
     * @param areaCode 地区编码
     * @return
     */
    List<AreaDTO> listByCode(String areaCode);

    /**
     * 地区字典子列表（不包含本级）
     *
     * @param areaCode 地区编码
     * @return
     */
    List<AreaDTO> listSubByCode(String areaCode);

}
